#!/usr/bin/env python3

"""Neural Network on a Commodore 64 --- Python 3 Clone

This software program is a Python 3 clone of the software J. Walker
released on September 4, 1987. It is available on the website

    https://www.fourmilab.ch/documents/commodore/BrainSim/

and implements a "complete neural network associative memory pattern
recogniser".

This program was written by P. Reiske (TU Berlin) and submitted to
S. Höltgen (HU Berlin) on March 16, 2022.
"""


import pickle
import platform
import random
import time
import os

# NOTE Since nowadays computers do no longer have easily accessible
# read-only memory fields to get characters' patterns from, the variabel
# `GROUND_TRUTH_PATTERN` is used to replace it.

from pattern import GROUND_TRUTH_PATTERN
from typing import List


CLEAR_COMMAND = 'clear' if platform.system() != 'Windows' else 'cls'


def sgn(x: int) -> int:
    """Signum function

    The signum function returns -1 if argument `x` has an value less than
    zero, 1 if argument `x` is greater than zero, and 0 if argument `x`
    equals zero.
    """

    if type(x) is not int:
        raise TypeError('Argument x must be of type int.')

    if x < 0:
        return -1

    if x == 0:
        return 0

    return 1


def update_display(left: List[int], right: List[int], status: str,
                   middle: str = '   ', sleep: bool = True) -> None:
    """Update user interface on command-line

    Keyword arguments:

    left : List[int] -- Pattern to be displayed in the left field.

    right : List[int] -- Pattern to be displayed in the right field.

    status : str -- Status information to be displayed in the bottom line.

    middle : str -- Up to three characters to be displayed in between the
    pattern fields.

    sleep : bool -- En-/ or disable 1 second sleep after updating the
    interface.
    """

    # Clear command-line interface
    os.system(CLEAR_COMMAND)

    # Configure command-line colors
    print('\033[95;45m', end='')

    # Print title line
    print('    NEURON NETWORK ASSOCIATIVE MEMORY   ')

    # Print empty lines
    for _ in range(3):
        print(40*' ')

    # Print top border of pattern fields
    print('     ┌──────┐          ┌──────┐         ')

    # Print pattern (and the field's left and right borders)
    for row in range(7):
        # Print left border of the left pattern fields
        print('     │', end='')

        # Print symbols in left pattern field
        for col in range(6):
            print('*' if left[6 * row + col] == 1 else ' ', end='')


        # Print right border of the left pattern field, left border of the
        # right pattern field, and (optional) the characters to be
        # displayed in between those two fields.
        if row == 4:
            print(f'│    {middle}   │', end='')
        else:
            print('│          │', end='')

        # Print symbols in right pattern field
        for col in range(6):
            print('*' if right[6 * row + col] == 1 else ' ', end='')

        # Print right border of left pattern field
        print('│         ')

    # Print bottom border of the pattern fields
    print('     └──────┘          └──────┘         ')

    # Print empty line
    print(40*' ')

    # Print list of available commands
    print('F1 - TEACH PATTERN     F2 - DUMP MATRIX ')
    print('F3 - RANDOMIZE PATTERN F4 - FORGET ALL  ')
    print('F5 - RECALL PATTERN    F6 - QUIT        ')
    print('F7 - DISC SAVE         F8 - DISC LOAD   ')

    # Print empty line
    print(40*' ')

    # Print available patterns
    print('A-Z, 0-9: LOAD PATTERN                  ')

    # Print empty line
    print(40*' ')

    # Print status line
    print(status, (39-len(status))*' ')

    # Print empty line
    print(40*' ')

    # Reset command-line colors
    print(39*' ', '\033[00m', '\r', end='')

    # Sleep for 1 second (optional)
    if sleep:
        time.sleep(1)


def main() -> None:
    f1 = [0 for _ in range(42)]
    f2 = [0 for _ in range(42)]
    m = [[0 for _ in range(42)] for _ in range(42)]

    while True:
        update_display(f2, f1, 'READY', sleep=False)

        # Wait for user command
        cmd = input('').strip().upper()

        if len(cmd) == 0:
            # Skip empty command
            continue
        elif len(cmd) == 1:  # LOAD PATTERN
            if cmd in GROUND_TRUTH_PATTERN:
                update_display(f2, f1, f'FETCH {cmd}')

                f1 = GROUND_TRUTH_PATTERN[cmd].copy()
            else:
                # Invalid user input, print command-line user interface
                # with error message.
                update_display(f2, f1, 'INVALID INPUT')
        elif len(cmd) == 2:
            if cmd[0] == 'F':
                if cmd[1] == '1':  # TEACH PATTERN
                    update_display(f2, f1, 'TRAINING')

                    for i in range(42):
                        for j in range(42):
                            m[i][j] += f1[i] * f1[j]
                elif cmd[1] == '2':  # DUMP MATRIX
                    os.system(CLEAR_COMMAND)

                    for i in range(23):
                        for j in range(40):
                            if m[i][j] < 0:
                                print('\033[31;45m', end='')
                            else:
                                print('\033[34;45m', end='')

                            print(chr(ord('0') + abs(m[i][j])), end='')

                        print()

                    print('\033[95;45mPRESS ENTER TO CONTINUE:',
                          '               \033[00m\r', end='')

                    input('')
                elif cmd[1] == '3':  # RANDOMIZE PATTERN
                    update_display(f2, f1, 'RANDOM')

                    for i in random.sample(list(range(42)), 4):
                        f1[i] *= -1
                elif cmd[1] == '4':  # FORGET ALL
                    update_display(f2, f1, 'FORGET')

                    for i in range(42):
                        f1[i] = 0
                        f2[i] = 0

                        for j in range(42):
                            m[i][j] = 0
                elif cmd[1] == '5':  # RECALL PATTERN
                    update_display(f2, f1, 'RECALL', ' = ')

                    for i in range(42):
                        f2[i] = f1[i]

                    update_display(f2, f1, 'RECALL', '==>')

                    for j in range(42):
                        v = 0

                        for i in range(42):
                            v += f1[i] * f2[j]

                        v = sgn(v)

                        if v != 0:
                            f2[j] = v

                    update_display(f2, f1, 'RECALL', '<==')

                    c = 0

                    for i in range(42):
                        v = 0

                        for j in range(42):
                            v += f2[j] * m[i][j]

                        v = sgn(v)

                        if v != 0 and v != f1[i]:
                            f1[i] = v
                            c = 1
                elif cmd[1] == '6':  # QUIT
                    update_display(f2, f1, 'QUIT', ' = ')

                    os.system(CLEAR_COMMAND)

                    return
                elif cmd[1] == '7':  # DISC SAVE
                    update_display(f2, f1, 'SAVE TO DISK: NEURON64.TXT')

                    with open('neuron64.txt', 'wb', 1_024) as file:
                        pickle.dump([m, f1, f2], file, 0)
                elif cmd[1] == '8':  # DISC LOAD
                    update_display(f2, f1, 'LOAD FROM DISK: NEURON64.TXT')

                    if os.path.isfile('neuron64.txt'):
                        with open('neuron64.txt', 'rb', 1_024) as file:
                            [m, f1, f2] = pickle.load(file)
                    else:
                        update_display(f2, f1, 'FILE NEURON64.TXT MISSING')
                else:
                    # Invalid user input, print command-line user interface
                    # with error message.
                    update_display(f2, f1, 'INVALID INPUT')
            else:
                # Invalid user input, print command-line user interface
                # with error message.
                update_display(f2, f1, 'INVALID INPUT')
        else:
            # Invalid user input, print command-line user interface
            # with error message.
            update_display(f2, f1, 'INVALID INPUT')


if __name__ == '__main__':
    main()
