"""Neural Network on a Commodore 64 --- Python 3 Clone

This software program is a Python 3 clone of the software J. Walker
released on September 4, 1987. It is available on the website

    https://www.fourmilab.ch/documents/commodore/BrainSim/

and implements a "complete neural network associative memory pattern
recogniser".

This program was written by P. Reiske (TU Berlin) and submitted to
S. Höltgen (HU Berlin) on March 16, 2022.
"""


GROUND_TRUTH_PATTERN = {
    '0': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1,  1,  1,
         1, -1,  1,  1, -1,  1,
         1,  1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    '1': [
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1,  1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
         1,  1,  1,  1,  1,  1,
    ],
    '2': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1,  1,  1,
        -1, -1, -1,  1,  1, -1,
        -1, -1,  1,  1, -1, -1,
        -1,  1,  1, -1, -1, -1,
         1,  1,  1,  1,  1,  1,
    ],
    '3': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
        -1, -1, -1, -1, -1,  1,
        -1, -1,  1,  1,  1, -1,
        -1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    '4': [
        -1, -1,  1,  1,  1, -1,
        -1,  1, -1, -1,  1, -1,
         1, -1, -1, -1,  1, -1,
         1,  1,  1, -1,  1,  1,
        -1, -1, -1, -1,  1, -1,
        -1, -1, -1, -1,  1, -1,
         1,  1,  1,  1,  1,  1,
    ],
    '5': [
         1,  1,  1,  1,  1,  1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1,  1,  1,  1,  1, -1,
        -1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    '6': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1, -1,
         1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    '7': [
         1,  1,  1,  1,  1,  1,
        -1, -1, -1, -1, -1,  1,
        -1, -1, -1, -1,  1, -1,
        -1, -1, -1,  1, -1, -1,
        -1, -1,  1, -1, -1, -1,
        -1,  1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
    ],
    '8': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    '9': [
       -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1,  1,
        -1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    'A': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1,  1,  1,  1,  1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
    ],
    'B': [
         1,  1,  1,  1, -1, -1,
         1, -1, -1, -1,  1, -1,
         1, -1, -1, -1,  1, -1,
         1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1,  1,  1,  1,  1, -1,
    ],
    'C': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    'D': [
         1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1,  1,  1,  1,  1, -1,
    ],
    'E': [
         1,  1,  1,  1,  1,  1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1,  1,  1,  1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1,  1,  1,  1,  1,  1,
    ],
    'F': [
         1,  1,  1,  1,  1,  1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1,  1,  1,  1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
    ],
    'G': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1,  1,  1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    'H': [
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1,  1,  1,  1,  1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
    ],
    'I': [
         1,  1,  1,  1,  1,  1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
         1,  1,  1,  1,  1,  1,
    ],
    'J': [
         1,  1,  1,  1,  1,  1,
        -1, -1, -1, -1, -1,  1,
        -1, -1, -1, -1, -1,  1,
        -1, -1, -1, -1, -1,  1,
        -1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    'K': [
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1,  1, -1,
         1, -1, -1,  1, -1, -1,
         1, -1,  1, -1, -1, -1,
         1,  1,  1, -1, -1, -1,
         1, -1, -1,  1, -1, -1,
         1, -1, -1, -1,  1, -1,
    ],
    'L': [
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1,  1,  1,  1,  1,  1,
    ],
    'M': [
         1, -1, -1, -1, -1,  1,
         1,  1, -1, -1,  1,  1,
         1, -1,  1,  1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
    ],
    'N': [
         1, -1, -1, -1, -1,  1,
         1,  1, -1, -1, -1,  1,
         1, -1,  1, -1, -1,  1,
         1, -1,  1,  1, -1,  1,
         1, -1, -1,  1, -1,  1,
         1, -1, -1, -1,  1,  1,
         1, -1, -1, -1, -1,  1,
    ],
    'O': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    'P': [
         1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
         1, -1, -1, -1, -1, -1,
    ],
    'Q': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1,  1, -1,  1,
         1, -1, -1, -1,  1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    'R': [
         1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1,  1,  1,  1,  1, -1,
         1, -1, -1,  1, -1, -1,
         1, -1, -1, -1,  1, -1,
         1, -1, -1, -1, -1,  1,
    ],
    'S': [
        -1,  1,  1,  1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1, -1,
        -1,  1,  1,  1,  1, -1,
        -1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    'T': [
         1,  1,  1,  1,  1,  1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
    ],
    'U': [
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1,  1,  1,  1, -1,
    ],
    'V': [
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1, -1, -1,  1, -1,
        -1,  1, -1, -1,  1, -1,
        -1, -1,  1,  1, -1, -1,
    ],
    'W': [
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
         1, -1,  1,  1, -1,  1,
         1, -1,  1,  1, -1,  1,
        -1,  1, -1, -1,  1, -1,
    ],
    'X': [
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1, -1, -1,  1, -1,
        -1, -1,  1,  1, -1, -1,
        -1,  1, -1, -1,  1, -1,
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
    ],
    'Y': [
         1, -1, -1, -1, -1,  1,
         1, -1, -1, -1, -1,  1,
        -1,  1, -1, -1,  1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
        -1, -1,  1,  1, -1, -1,
    ],
    'Z': [
         1,  1,  1,  1,  1,  1,
        -1, -1, -1, -1,  1, -1,
        -1, -1, -1,  1,  1, -1,
        -1, -1,  1,  1, -1, -1,
        -1,  1,  1, -1, -1, -1,
        -1,  1, -1, -1, -1, -1,
         1,  1,  1,  1,  1,  1,
    ]
}
